/**
 * Copyright (c) 2000-present Liferay, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of the Liferay Enterprise
 * Subscription License ("License"). You may not use this file except in
 * compliance with the License. You can obtain a copy of the License by
 * contacting Liferay, Inc. See the License for the specific language governing
 * permissions and limitations under the License, including but not limited to
 * distribution rights of the Software.
 *
 *
 *
 */

package com.liferay.segments.context.vocabulary.internal.configuration.persistence.listener;

import com.liferay.petra.string.StringBundler;
import com.liferay.portal.configuration.persistence.listener.ConfigurationModelListener;
import com.liferay.portal.configuration.persistence.listener.ConfigurationModelListenerException;
import com.liferay.portal.kernel.util.LocaleThreadLocal;
import com.liferay.portal.kernel.util.ResourceBundleUtil;
import com.liferay.portal.kernel.util.Validator;

import java.util.Dictionary;
import java.util.Objects;
import java.util.ResourceBundle;

import org.osgi.service.cm.Configuration;
import org.osgi.service.cm.ConfigurationAdmin;
import org.osgi.service.component.annotations.Reference;

/**
 * @author Janis Zhang
 */
public abstract class BaseConfigurationModelListener
	implements ConfigurationModelListener {

	@Override
	public void onBeforeSave(String pid, Dictionary<String, Object> properties)
		throws ConfigurationModelListenerException {

		String entityFieldName = String.valueOf(
			properties.get("entityFieldName"));

		ResourceBundle resourceBundle = ResourceBundleUtil.getBundle(
			"content.Language", LocaleThreadLocal.getThemeDisplayLocale(),
			getClass());

		if (Validator.isNull(entityFieldName)) {
			throw new ConfigurationModelListenerException(
				ResourceBundleUtil.getString(
					resourceBundle,
					"please-enter-a-valid-session-property-name"),
				getConfigurationClass(), getClass(), properties);
		}

		if (_isDefined(
				String.valueOf(properties.get("assetVocabularyName")),
				String.valueOf(properties.get("companyId")), entityFieldName)) {

			throw new DuplicatedSegmentsContextVocabularyConfigurationModelListenerException(
				ResourceBundleUtil.getString(
					resourceBundle,
					"this-field-is-already-linked-to-one-vocabulary"),
				getConfigurationClass(), getClass(), properties);
		}
	}

	protected abstract Class<?> getConfigurationClass();

	@Reference
	protected ConfigurationAdmin configurationAdmin;

	private boolean _isDefined(
			String assetVocabularyName, String companyId,
			String entityFieldName)
		throws ConfigurationModelListenerException {

		try {
			Configuration[] configurations =
				configurationAdmin.listConfigurations(
					StringBundler.concat(
						"(", ConfigurationAdmin.SERVICE_FACTORYPID, "=",
						getConfigurationClass().getCanonicalName(), ")"));

			if (configurations == null) {
				return false;
			}

			for (Configuration configuration : configurations) {
				Dictionary<String, Object> properties =
					configuration.getProperties();

				if ((Objects.equals(
						assetVocabularyName,
						properties.get("assetVocabularyName")) &&
					 Objects.equals(
						 entityFieldName, properties.get("entityFieldName"))) ||
					(Objects.equals(
						companyId,
						String.valueOf(properties.get("companyId"))) &&
					 Objects.equals(
						 entityFieldName, properties.get("entityFieldName")))) {

					return true;
				}
			}

			return false;
		}
		catch (Exception exception) {
			throw new ConfigurationModelListenerException(
				exception.getMessage(), getConfigurationClass(), getClass(),
				null);
		}
	}

}